@@ -213,7 +213,7 @@ class _LlamaModel:
 
     NOTE: For stability it's recommended you use the Llama class instead."""
 
-    _llama_free_model = llama_cpp._lib.llama_free_model  # type: ignore
+    _llama_free_model = None
 
     def __init__(
         self,
@@ -226,6 +226,8 @@ class _LlamaModel:
         self.params = params
         self.verbose = verbose
 
+        self._llama_free_model = llama_cpp._lib.llama_free_model  # type: ignore
+
         if not os.path.exists(path_model):
             raise ValueError(f"Model path does not exist: {path_model}")
 
@@ -236,7 +238,7 @@ class _LlamaModel:
 
     def __del__(self):
         with suppress_stdout_stderr(disable=self.verbose):
-            if self.model is not None:
+            if self.model is not None and self._llama_free_model is not None:
                 self._llama_free_model(self.model)
                 self.model = None
 
@@ -396,7 +398,7 @@ class _LlamaContext:
 
     NOTE: For stability it's recommended you use the Llama class instead."""
 
-    _llama_free = llama_cpp._lib.llama_free  # type: ignore
+    _llama_free = None
 
     def __init__(
         self,
@@ -409,6 +411,8 @@ class _LlamaContext:
         self.params = params
         self.verbose = verbose
 
+        self._llama_free = llama_cpp._lib.llama_free  # type: ignore
+
         with suppress_stdout_stderr(disable=self.verbose):
             self.ctx = llama_cpp.llama_new_context_with_model(
                 self.model.model, self.params
@@ -416,7 +420,7 @@ class _LlamaContext:
 
     def __del__(self):
         with suppress_stdout_stderr(disable=self.verbose):
-            if self.ctx is not None:
+            if self.ctx is not None and self._llama_free is not None:
                 self._llama_free(self.ctx)
                 self.ctx = None
 
@@ -645,7 +649,7 @@ class _LlamaContext:
 
 
 class _LlamaBatch:
-    _llama_batch_free = llama_cpp._lib.llama_batch_free  # type: ignore
+    _llama_batch_free = None
 
     def __init__(
         self, *, n_tokens: int, embd: int, n_seq_max: int, verbose: bool = True
@@ -655,6 +659,8 @@ class _LlamaBatch:
         self.n_seq_max = n_seq_max
         self.verbose = verbose
 
+        self._llama_batch_free = llama_cpp._lib.llama_batch_free  # type: ignore
+
         with suppress_stdout_stderr(disable=self.verbose):
             self.batch = llama_cpp.llama_batch_init(
                 self.n_tokens, self.embd, self.n_seq_max
@@ -662,7 +668,7 @@ class _LlamaBatch:
 
     def __del__(self):
         with suppress_stdout_stderr(disable=self.verbose):
-            if self.batch is not None:
+            if self.batch is not None and self._llama_batch_free is not None:
                 self._llama_batch_free(self.batch)
                 self.batch = None
 
@@ -992,6 +998,15 @@ class Llama:
         """
         self.cache = cache
 
+    def set_seed(self, seed: int):
+        """Set the random seed.
+
+        Args:
+            seed: The random seed.
+        """
+        assert self._ctx.ctx is not None
+        llama_cpp.llama_set_rng_seed(self._ctx.ctx, seed)
+
     def reset(self):
         """Reset the model state."""
         self.n_tokens = 0
@@ -1004,27 +1019,26 @@ class Llama:
         """
         assert self._ctx.ctx is not None
         assert self._batch.batch is not None
-        n_ctx = self._n_ctx
+        self._ctx.kv_cache_seq_rm(-1, self.n_tokens, -1)
         for i in range(0, len(tokens), self.n_batch):
             batch = tokens[i : min(len(tokens), i + self.n_batch)]
-            n_past = min(n_ctx - len(batch), self.n_tokens)
+            n_past = self.n_tokens
             n_tokens = len(batch)
-            self._ctx.kv_cache_seq_rm(-1, n_past, -1)
             self._batch.set_batch(
                 batch=batch, n_past=n_past, logits_all=self.context_params.logits_all
             )
             self._ctx.decode(self._batch)
             # Save tokens
-            self.input_ids[self.n_tokens : self.n_tokens + n_tokens] = batch
+            self.input_ids[n_past : n_past + n_tokens] = batch
             # Save logits
-            rows = n_tokens if self.context_params.logits_all else 1
+            rows = n_tokens
             cols = self._n_vocab
             offset = (
                 0 if self.context_params.logits_all else n_tokens - 1
             )  # NOTE: Only save the last token logits if logits_all is False
-            self.scores[self.n_tokens + offset : self.n_tokens + n_tokens, :].reshape(
+            self.scores[n_past + offset : n_past + n_tokens, :].reshape(
                 -1
-            )[:] = self._ctx.get_logits()[: rows * cols]
+            )[:] = self._ctx.get_logits()[offset * cols: rows * cols]
             # Update n_tokens
             self.n_tokens += n_tokens
 
@@ -1281,7 +1295,7 @@ class Llama:
         self,
         prompt: Union[str, List[int]],
         suffix: Optional[str] = None,
-        max_tokens: int = 16,
+        max_tokens: Optional[int] = 16,
         temperature: float = 0.8,
         top_p: float = 0.95,
         logprobs: Optional[int] = None,
@@ -1312,10 +1326,14 @@ class Llama:
         completion_tokens: List[int] = []
         # Add blank space to start of prompt to match OG llama tokenizer
         prompt_tokens: List[int] = (
-            self.tokenize(prompt.encode("utf-8"), special=True)
-            if prompt != ""
-            else [self.token_bos()]
-        ) if isinstance(prompt, str) else prompt
+            (
+                self.tokenize(prompt.encode("utf-8"), special=True)
+                if prompt != ""
+                else [self.token_bos()]
+            )
+            if isinstance(prompt, str)
+            else prompt
+        )
         text: bytes = b""
         returned_tokens: int = 0
         stop = (
@@ -1331,7 +1349,7 @@ class Llama:
                 f"Requested tokens ({len(prompt_tokens)}) exceed context window of {llama_cpp.llama_n_ctx(self.ctx)}"
             )
 
-        if max_tokens <= 0:
+        if max_tokens is None or max_tokens <= 0:
             # Unlimited, depending on n_ctx.
             max_tokens = self._n_ctx - len(prompt_tokens)
 
@@ -1368,7 +1386,7 @@ class Llama:
             except KeyError:
                 if self.verbose:
                     print("Llama._create_completion: cache miss", file=sys.stderr)
-        
+
         if seed is not None:
             self._ctx.set_rng_seed(seed)
 
@@ -1743,7 +1761,7 @@ class Llama:
         self,
         prompt: Union[str, List[int]],
         suffix: Optional[str] = None,
-        max_tokens: int = 128,
+        max_tokens: Optional[int] = 16,
         temperature: float = 0.8,
         top_p: float = 0.95,
         logprobs: Optional[int] = None,
@@ -1769,7 +1787,7 @@ class Llama:
         Args:
             prompt: The prompt to generate text from.
             suffix: A suffix to append to the generated text. If None, no suffix is appended.
-            max_tokens: The maximum number of tokens to generate. If max_tokens <= 0, the maximum number of tokens to generate is unlimited and depends on n_ctx.
+            max_tokens: The maximum number of tokens to generate. If max_tokens <= 0 or None, the maximum number of tokens to generate is unlimited and depends on n_ctx.
             temperature: The temperature to use for sampling.
             top_p: The top-p value to use for sampling.
             logprobs: The number of logprobs to return. If None, no logprobs are returned.
@@ -1902,7 +1920,7 @@ class Llama:
         stop: Optional[Union[str, List[str]]] = [],
         seed: Optional[int] = None,
         response_format: Optional[ChatCompletionRequestResponseFormat] = None,
-        max_tokens: int = 256,
+        max_tokens: Optional[int] = None,
         presence_penalty: float = 0.0,
         frequency_penalty: float = 0.0,
         repeat_penalty: float = 1.1,
@@ -1925,7 +1943,7 @@ class Llama:
             top_k: The top-k value to use for sampling.
             stream: Whether to stream the results.
             stop: A list of strings to stop generation when encountered.
-            max_tokens: The maximum number of tokens to generate. If max_tokens <= 0, the maximum number of tokens to generate is unlimited and depends on n_ctx.
+            max_tokens: The maximum number of tokens to generate. If max_tokens <= 0 or None, the maximum number of tokens to generate is unlimited and depends on n_ctx.
             repeat_penalty: The penalty to apply to repeated tokens.
 
         Returns:
