@@ -384,7 +384,7 @@ def create_app(settings: Optional[Settings] = None):
     chat_handler = None
     if settings.chat_format == "llava-1-5":
         assert settings.clip_model_path is not None
-        chat_handler = llama_cpp.llama_chat_format.Llava15ChatHandler(clip_model_path=settings.clip_model_path)
+        chat_handler = llama_cpp.llama_chat_format.Llava15ChatHandler(clip_model_path=settings.clip_model_path, verbose=settings.verbose)
     ##
 
     llama = llama_cpp.Llama(
@@ -783,7 +783,9 @@ class CreateChatCompletionRequest(BaseModel):
         default=None,
         description="A tool to apply to the generated completions.",
     ) # TODO: verify
-    max_tokens: int = max_tokens_field
+    max_tokens: Optional[int] = Field(
+        default=None, description="The maximum number of tokens to generate. Defaults to inf"
+    )
     temperature: float = temperature_field
     top_p: float = top_p_field
     stop: Optional[List[str]] = stop_field
